@@ -32,8 +32,8 @@ labeling_list <- function(gp = gpar(),
     if (is.table(d))
       d <- dimnames(d)
     ld <- length(d)
-    labeling_text(labels = FALSE, varnames = varnames)(d, split_vertical, condvars)
-    seekViewport("marginBottom")
+    labeling_border(labels = FALSE, varnames = varnames)(d, split_vertical, condvars)
+    seekViewport("margin_bottom")
     pos <- unit(switch(pos, left = 0, center = 0.5, 1) / cols, "npc")
     ind <- split(seq(ld), rep.int(seq(cols), ceiling(ld / cols))[seq(ld)])
     
@@ -57,7 +57,7 @@ labeling_conditional <- function(...)
       d <- dimnames(d)
     v <- rep.int(TRUE, length(d))
     v[seq(condvars)] <- FALSE
-    labeling_text(labels = !v, ...)(d, split_vertical, condvars)
+    labeling_border(labels = !v, ...)(d, split_vertical, condvars)
     labeling_cells(labels = v, ...)(d, split_vertical, condvars)
   }
 class(labeling_conditional) <- "panel_generator"
@@ -97,8 +97,8 @@ labeling_cells <- function(labels = TRUE, varnames = TRUE,
       for (labind in seq(along = n)) {
         lab <- c(labs, n[labind])
         names(lab) <- names(d)[1:vind]
-        mlab <- paste("cell", paste(dn[1:vind], lab, sep = ".", collapse = ".."),
-                      sep = "..")
+        mlab <- paste("cell:", paste(dn[1:vind], lab, sep = "=", collapse = ","),
+                      sep = "")
 
         if (vind < ld)
           split(vind + 1, lab)
@@ -132,23 +132,24 @@ labeling_cells <- function(labels = TRUE, varnames = TRUE,
 }
 class(labeling_cells) <- "panel_generator"
 
-labeling_text <- function(labels = TRUE, varnames = labels,
-                          tl_labels = NULL, tl_varnames = NULL, 
-                          gp_labels = gpar(fontsize = 12),
-                          gp_varnames = gpar(fontsize = 12, fontface = 2),
-                          rot_labels = c(0, 90, 0, 90),
-                          rot_varnames = c(0, 90, 0, 90),
-                          pos_labels = "center", pos_varnames = "center",
-                          just_labels = "center", just_varnames = pos_varnames,
-                          boxes = FALSE, fill_boxes = NULL,
-                          offset = c(0, 0, 0, 0),
+labeling_border <- function(labels = TRUE, varnames = labels,
+                            set_labels = NULL, set_varnames = NULL,
+                            tl_labels = NULL, tl_varnames = NULL, 
+                            gp_labels = gpar(fontsize = 12),
+                            gp_varnames = gpar(fontsize = 12, fontface = 2),
+                            rot_labels = c(0, 90, 0, 90),
+                            rot_varnames = c(0, 90, 0, 90),
+                            pos_labels = "center", pos_varnames = "center",
+                            just_labels = "center", just_varnames = pos_varnames,
+                            boxes = FALSE, fill_boxes = FALSE,
+                            offset = c(0, 0, 0, 0),
                           
-                          labbl_varnames = NULL,
-                          labels_varnames = FALSE, sep = ": ",
+                            labbl_varnames = NULL,
+                            labels_varnames = FALSE, sep = ": ",
                           
-                          abbreviate = FALSE, rep = TRUE,
-                          clip = FALSE, ...
-                          )
+                            abbreviate = FALSE, rep = TRUE,
+                            clip = FALSE, ...
+                            )
   function(d, split_vertical, condvars) {
     if (is.table(d))
       d <- dimnames(d)
@@ -231,11 +232,29 @@ labeling_text <- function(labels = TRUE, varnames = labels,
     boxes <- pexpand(boxes, ld, FALSE, dn)
 
     ## fill_boxes
-    if (is.null(fill_boxes))
-      fill_boxes <- lapply(sapply(d, length),
-                           function(i) gray(0.3 + 0.4 * rev(seq(i)) / i))
-    else
-      fill_boxes <- pexpand(fill_boxes, ld, "grey", dn)
+    dnl <- sapply(d, length)
+    fill_boxes <- if (is.atomic(fill_boxes)) {
+      fill_boxes <- if (is.logical(fill_boxes))
+        ifelse(pexpand(fill_boxes, ld, FALSE, dn), "grey", NA)
+      else
+        pexpand(fill_boxes, ld, "grey", dn)
+      col <- rgb2hsv(col2rgb(fill_boxes))
+      lapply(seq(along.with = dnl),
+             function(i) if (is.na(fill_boxes[i])) "white" else
+                         hsv(h = col["h",i],
+                             s = col["s",i],
+                             v = seq(from = col["v",i],
+                                     to = 0.5 * col["v",i],
+                                     length = dnl[i])
+                             )
+             )
+    } else {
+      fill_boxes <- pexpand(fill_boxes, ld, "white", dn)
+      lapply(seq(ld),
+             function(i) pexpand(fill_boxes[[i]], dnl[i], "white", d[[i]])
+             )
+    }
+    
 
     ## precompute spaces
     lsp <- tsp <- bsp <- rsp <- 0
@@ -255,21 +274,25 @@ labeling_text <- function(labels = TRUE, varnames = labels,
     ## varnames in the outer margin  
       ## compute axis names
       tt <- bt <- lt <- rt <- ""
-      for (i in seq(along = dn))
+      for (i in seq(along = dn)) {
+        var <- if (!is.null(set_varnames) && !is.na(set_varnames[dn[i]]))
+          set_varnames[dn[i]]
+        else
+          dn[i]
         if (varnames[i]) {
           if (split_vertical[i]) {
             if (tl_varnames[i])
-              tt <- paste(tt, dn[i], sep = if (tt == "") "" else " / ")
+              tt <- paste(tt, var, sep = if (tt == "") "" else " / ")
             else
-              bt <- paste(bt, dn[i], sep = if (bt == "") "" else " / ")
+              bt <- paste(bt, var, sep = if (bt == "") "" else " / ")
           } else {
             if (tl_varnames[i])
-              lt <- paste(lt, dn[i], sep = if (lt == "") "" else " / ")
+              lt <- paste(lt, var, sep = if (lt == "") "" else " / ")
             else
-              rt <- paste(rt, dn[i], sep = if (rt == "") "" else " / ")
+              rt <- paste(rt, var, sep = if (rt == "") "" else " / ")
           }
         }
-
+      }
       ## draw axis names
       if (tt != "")
         grid.text(tt, y = unit(1, "npc") + unit(tsp + 1, "lines") + offset[1],
@@ -289,27 +312,31 @@ labeling_text <- function(labels = TRUE, varnames = labels,
                   rot = rot_varnames[2], just = just_varnames[2], gp = gp_varnames[[2]])
     } else {
     ## varnames beneath labels
-      for (i in seq(along = dn))
+      for (i in seq(along = dn)) {
+        var <- if (!is.null(set_varnames) && !is.na(set_varnames[dn[i]]))
+          set_varnames[dn[i]]
+        else
+          dn[i]
         if (varnames[i]) {
           if (split_vertical[i]) {
             if (tl_labels[i]) {
               if (labbl_varnames[i]) {
-                grid.text(dn[i],
+                grid.text(var,
                           y = unit(1, "npc") + unit(1 + tsp - labsp[i], "lines") + offset[1],
                           x = unit(-0.5, "lines"),
                           just = "right", gp = gpar(fontface = 2))
               } else {
-                grid.text(dn[i], y = unit(1, "npc") + unit(1 + tsp - labsp[i], "lines") + offset[1],
+                grid.text(var, y = unit(1, "npc") + unit(1 + tsp - labsp[i], "lines") + offset[1],
                           x = unit(1, "npc") + unit(0.5, "lines"),
                           just = "left", gp = gpar(fontface = 2))
               }
             } else {
               if (labbl_varnames[i]) {
-                grid.text(dn[i], y = unit(labsp[i], "lines") + -1 * offset[3],
+                grid.text(var, y = unit(labsp[i], "lines") + -1 * offset[3],
                           x = unit(-0.5, "lines"), just = "right",
                           gp = gpar(fontface = 2))
               } else {
-                grid.text(dn[i], y = unit(labsp[i], "lines") + -1 * offset[3],
+                grid.text(var, y = unit(labsp[i], "lines") + -1 * offset[3],
                           x = unit(1, "npc") + unit(0.5, "lines"),
                           just = "left", gp = gpar(fontface = 2))
               }
@@ -317,31 +344,32 @@ labeling_text <- function(labels = TRUE, varnames = labels,
           } else {
             if (tl_labels[i]) {
               if (labbl_varnames[i]) {
-                grid.text(dn[i], x = unit(lsp - 1 - labsp[i], "lines") + -1 * offset[4],
+                grid.text(var, x = unit(lsp - 1 - labsp[i], "lines") + -1 * offset[4],
                           y = unit(-0.5, "lines"), just = "right", rot = 90,
                           gp = gpar(fontface = 2))
               } else {
-                grid.text(dn[i], x = unit(lsp - 1 - labsp[i], "lines") + -1 * offset[4],
+                grid.text(var, x = unit(lsp - 1 - labsp[i], "lines") + -1 * offset[4],
                           y = unit(1, "npc") + unit(0.5, "lines"),
                           just = "left", rot = 90, gp = gpar(fontface = 2))
               }
             } else {
               if (labbl_varnames[i]) {
-                grid.text(dn[i], x = unit(1, "npc") + unit(labsp[i], "lines") + offset[2],
+                grid.text(var, x = unit(1, "npc") + unit(labsp[i], "lines") + offset[2],
                           y = unit(-0.5, "lines"),
                           just = "right", rot = 90, gp = gpar(fontface = 2))
               } else {
-                grid.text(dn[i], x = unit(1, "npc") + unit(labsp[i], "lines") + offset[2],
+                grid.text(var, x = unit(1, "npc") + unit(labsp[i], "lines") + offset[2],
                           y = unit(1, "npc") + unit(0.5, "lines"),
                           just = "left", rot = 90, gp = gpar(fontface = 2))
               }
             }
           }
         }
+      }
     }
 
     ## draw labels
-    split <- function(vind = 1, root = "cell",
+    split <- function(vind = 1, root = "cell:",
                       left = TRUE, right = TRUE, top = TRUE, bottom = TRUE) {
       n <- d[[vind]]
       vl <- length(n)
@@ -350,11 +378,17 @@ labeling_text <- function(labels = TRUE, varnames = labels,
       if (!sp) labseq <- rev(labseq)
       
       for (labind in labseq) {
-        mlab <- paste(root, "", dn[vind], n[labind], sep = ".")
+        mlab <- paste(root, dn[vind], "=", n[labind], sep = "")
         if (labels[vind] && (rep[vind] || !printed[[vind]][labind])) {
-          lab <- labs[[vind]][labind]
+          lab <- if (!is.null(set_labels) && !is.null(set_labels[[dn[vind]]]))
+            set_labels[[dn[vind]]][labind]
+          else
+            labs[[vind]][labind]
           if (labels_varnames[vind])
-            lab <- paste(dn[vind], lab, sep = sep)
+            lab <- if (!is.null(set_varnames) && !is.na(set_varnames[dn[vind]]))
+              paste(set_varnames[dn[vind]], lab, sep = sep)
+          else
+              paste(dn[vind], lab, sep = sep)
           if (sp) {
             if (tl_labels[vind]) {
               if (top) {
@@ -451,7 +485,7 @@ labeling_text <- function(labels = TRUE, varnames = labels,
           }
         }
         
-        if (vind < ld) Recall(vind + 1, mlab,
+        if (vind < ld) Recall(vind + 1, paste(mlab, ",", sep = ""),
                               if (sp) left && labind == 1 else left,
                               if (sp) right && labind == vl else right,
                               if (!sp) top && labind == 1 else top,
@@ -461,14 +495,14 @@ labeling_text <- function(labels = TRUE, varnames = labels,
     split()
     
   }
-class(labeling_text) <- "panel_generator"
+class(labeling_border) <- "panel_generator"
 
 labeling_doubledecker <- function(lab_pos = c("bottom", "top"), ...) {
   lab_pos <- match.arg(lab_pos)
   function(d, split_vertical, condvars) {
     if (is.table(d))
       d <- dimnames(d)
-    labeling_text(boxes = c(rep.int(TRUE, length(d) - 1), FALSE),
+    labeling_border(boxes = c(rep.int(TRUE, length(d) - 1), FALSE),
                   clip = c(rep.int(TRUE, length(d) - 1), FALSE),
                   labbl_varnames = FALSE,
                   rot_labels = rep.int(0, 4),
@@ -478,7 +512,7 @@ labeling_doubledecker <- function(lab_pos = c("bottom", "top"), ...) {
                   offset = c(0, -0.6, 0, 0),
                   tl_labels = c(rep.int(lab_pos== "top", length(d) - 1), FALSE)
                   )(d, split_vertical, condvars)
-    seekViewport("marginRight")
+    seekViewport("margin_right")
     grid.text(names(d)[length(d)],
               x = unit(0.5, "lines"), y = unit(1, "npc"), just = c("left","top"),
               gp = gpar(fontface = 2))
@@ -488,14 +522,14 @@ class(labeling_doubledecker) <- "panel_generator"
 
 labeling_left <- function(tl_labels = TRUE, clip = TRUE, pos_varnames = "left",
                         pos_labels = "left", just_labels = "left", ...)
-  labeling_text(tl_labels = tl_labels, clip = clip, pos_varnames = pos_varnames,
+  labeling_border(tl_labels = tl_labels, clip = clip, pos_varnames = pos_varnames,
               pos_labels = pos_labels, just_labels = just_labels, ...)
 class(labeling_left) <- "panel_generator"
 
 labeling_cboxed <- function(tl_labels = TRUE, boxes = TRUE, clip = TRUE, pos_labels = "center", ...)
-  labeling_text(tl_labels = tl_labels, boxes = boxes, clip = clip, pos_labels = pos_labels, ...)
+  labeling_border(tl_labels = tl_labels, boxes = boxes, clip = clip, pos_labels = pos_labels, ...)
 class(labeling_cboxed) <- "panel_generator"
 
 labeling_lboxed <- function(tl_labels = FALSE, boxes = TRUE, clip = TRUE, pos_labels = "left", just_labels = "left", labbl_varnames = FALSE, ...)
-  labeling_text(tl_labels = tl_labels, boxes = boxes, clip = clip, pos_labels = pos_labels, labbl_varnames = labbl_varnames, just_labels = just_labels, ...)
+  labeling_border(tl_labels = tl_labels, boxes = boxes, clip = clip, pos_labels = pos_labels, labbl_varnames = labbl_varnames, just_labels = just_labels, ...)
 class(labeling_lboxed) <- "panel_generator"
